@@ -1,3 +1,6 @@
+# TODO: [#33] Fix pylint errors
+# pylint: disable=unsubscriptable-object,too-many-function-args
+
 # Standard library imports
 import concurrent.futures
 import copy
@@ -10,7 +13,7 @@ import time
 import typing
 
 # Related third party imports
-import dpath.util
+import dpath
 import docker
 import requests
 import requests_file
@@ -56,7 +59,7 @@ INVALID_JOB_NAMES = ["after_script",
 TEMPLATE_URL = "https://gitlab.com/gitlab-org/gitlab/-/raw/master/lib/gitlab/ci/templates/"
 
 # Global variables
-__version__ = "1.1"
+__version__ = "1.4"
 
 # Force output to utf-8
 sys.stdout.reconfigure(encoding="utf-8")
@@ -84,7 +87,7 @@ class GitlabCI:
 
                 del gitlab_ci['include']
                 for include in includes:
-                    dpath.util.merge(gitlab_ci, include)
+                    dpath.merge(gitlab_ci, include)
 
         # Expand the extends
         # NOTE: Not supporting an array of extends
@@ -93,7 +96,7 @@ class GitlabCI:
                 job_parameters = gitlab_ci[job_name]
                 extended_job_name = job_parameters.pop('extends')
                 extended_job = copy.deepcopy(gitlab_ci[extended_job_name])
-                dpath.util.merge(extended_job, job_parameters, flags=dpath.util.MERGE_REPLACE)
+                dpath.merge(extended_job, job_parameters, flags=dpath.types.MergeType.REPLACE)
                 gitlab_ci[job_name] = extended_job
 
         # Keep the expanded result
@@ -145,15 +148,15 @@ class GitlabCI:
 
         elif 'local' in detail:
             if detail.get('_included_from', ""):
-                url = detail['_included_from'] + detail['local']
+                url = detail['_included_from'].rstrip("/") + "/" + detail['local'].lstrip("/")
             else:
-                url = "file://" + os.getcwd() + detail['local']
+                url = "file://" + os.getcwd().rstrip("/") + "/" + detail['local'].lstrip("/")
 
         elif 'remote' in detail:
             url = detail['remote']
 
         elif 'template' in detail:
-            url = self.template_url + detail['template']
+            url = self.template_url.rstrip("/") + "/" + detail['template'].lstrip("/")
 
         else:
             raise RuntimeError("Unknown include type")
@@ -261,9 +264,11 @@ class GitlabJob:
             if isinstance(service, str):
                 image_name = service
                 container_name = service.split(":")[0]
+                variables = None
             else:
                 image_name = service['name']
                 container_name = service['alias']
+                variables = service.get("variables", None)
 
             links[container_name] = None
 
@@ -283,6 +288,7 @@ class GitlabJob:
             print("Starting service {}... ".format(container_name), end="", flush=True)
             docker_server.containers.run(image_name,
                                          detach=True,
+                                         environment=variables,
                                          name=container_name,
                                          privileged=bool(image_name == "docker:dind"),
                                          remove=True,
@@ -293,7 +299,7 @@ class GitlabJob:
 
     def run(self,
             extra_variables: dict = None,
-            extra_volumes: dict = None,
+            extra_volumes: list = None,
             pull_image: bool = True,
             user: str = "",
             ) -> int:
@@ -307,8 +313,8 @@ class GitlabJob:
 
         # Set the variables
         variables = copy.deepcopy(GITLAB_VARIABLES)
-        dpath.util.merge(variables, self.variables)
-        dpath.util.merge(variables, extra_variables)
+        dpath.merge(variables, self.variables)
+        dpath.merge(variables, extra_variables)
         for name, value in variables.items():
             if not isinstance(value, str):
                 variables[name] = str(value)
@@ -350,14 +356,12 @@ class GitlabJob:
         # Set volumes to mount
         docker_dir = ensure_unix_path(os.path.expanduser("~/.docker"))
         project_dir = variables['CI_PROJECT_DIR']
-        volumes = {ensure_unix_path(script.name): {'bind': "/tmp/job.sh", 'mode': 'ro'},
-                   docker_dir: {'bind': "/root/.docker", 'mode': 'ro'},
-                   project_dir: {'bind': project_dir, 'mode': 'rw'},
-                   '/var/run/docker.sock': {'bind': "/var/run/docker.sock",
-                                            'mode': 'rw',
-                                            },
-                   }
-        dpath.util.merge(volumes, extra_volumes)
+        volumes = [f"{ensure_unix_path(script.name)}:/tmp/job.sh:ro",
+                   f"{docker_dir}:/root/.docker:ro",
+                   f"{project_dir}:{project_dir}:rw",
+                   "/var/run/docker.sock:/var/run/docker.sock:rw",
+                   ]
+        volumes += extra_volumes if extra_volumes is not None else []
 
         # Set the command
         # NOTE: Emulating GitLab's step_script (https://gitlab.com/gitlab-org/gitlab-runner/-/blob/79911f89b70dcbb1f16ca7a1242d5919aef52c4a/shells/bash.go#L17)     # pylint: disable=line-too-long
